from errbot import BotPlugin, botcmd

import json
import requests

import functools

class Cloud(BotPlugin):
    """
    This is a very basic plugin to try out your new installation and get you started.
    Feel free to tweak me to experiment with Errbot.
    You can find me in your init directory in the subdirectory plugins.
    """
    
    api_root = "http://10.42.16.120:8080"
    namespace = "default"

    @botcmd(split_args_with=None)
    def k8s_status(self, msg, args):  # a command callable with !tryme
        """
        Execute to check if Errbot responds to command.
        Feel free to tweak me to experiment with Errbot.
        You can find me in your init directory in the subdirectory plugins.
        """
        
        n_str = requests.get(self.api_root + "/api/v1/nodes")
        n = n_str.json()
        
        s_str = requests.get(self.api_root + "/api/v1/services")
        s = s_str.json()
        
        d_str = requests.get(self.api_root + "/apis/extensions/v1beta1/deployments")
        d = d_str.json()
        
        i_str = requests.get(self.api_root + "/apis/extensions/v1beta1/ingresses")
        i = i_str.json()
        
        s_items = filter(lambda s: s['metadata']['namespace'] == args[0], s['items']) if len(args) else s['items']
        d_items = filter(lambda d: d['metadata']['namespace'] == args[0], d['items']) if len(args) else d['items']
        i_items = filter(lambda i: i['metadata']['namespace'] == args[0], i['items']) if len(args) else i['items']
        
        n_txt = 'Nodes:\n'
        for node in n['items']:
            n_txt += ' - Node ' + node['metadata']['name']
            n_txt += ' - Capacity ' +  node['status']['capacity']['cpu'] + 'cpu / ' + node['status']['capacity']['memory']
            n_txt += ' - OS ' + node['status']['nodeInfo']['osImage'] + '/' + node['status']['nodeInfo']['kernelVersion']
            n_txt += '\n'
            
        yield n_txt
        
        s_txt = 'Services: \n'
        for service in s_items:
            s_txt += ' - Service ' + service['metadata']['name'] + '(' + service['metadata']['namespace'] + ')'
            for s_p in service['spec']['ports']:
                s_txt += ' - Port ' + (s_p['name'] if 'name' in s_p else 'Unnamed') + ' -> ' + s_p['protocol']+ '@' + str(s_p['port']) + '/' + str(s_p['targetPort'])
            s_txt += ' - IP ' +  service['spec']['clusterIP']
            s_txt += '\n'
        
        yield s_txt
        
        d_txt = 'Deployments: \n'
        for deployment in d_items:
            d_txt += ' - Deployment ' + deployment['metadata']['name'] + '(' + deployment['metadata']['namespace'] + ')'
            rep = deployment['spec']['replicas'] if 'replicas' in deployment['spec'] else 'Unknown'
            containers = functools.reduce(lambda a, b: a + ' ' + b['name'] + '(' + b['image'] + ')', deployment['spec']['template']['spec']['containers'], '')
            d_txt += ' ' +  str(rep) + ' replicas of ' + containers
            
        yield d_txt
        
        i_txt = 'Ingresses: \n'
        for ingress in i_items:
            i_txt += ' - Ingress ' + ingress['metadata']['name'] + '(' + ingress['metadata']['namespace'] + ')\n'
            if 'rules' in ingress['spec']:
                for ingress_rule in ingress['spec']['rules']:
                    i_txt += '        ' + ingress_rule['host'] + ' -> ' + self.print_ingress_rule(ingress_rule)
                    i_txt += '\n'
            
        yield i_txt
        
        
    def print_ingress_rule(self, ingress_rule):
        rule_txt = ''
        for rule in ingress_rule['http']['paths']:
            rule_txt += rule['backend']['serviceName'] + ':' + rule['backend']['servicePort'] + ', '
        
        return rule_txt

def main():
    cloud = Cloud(None)
    res = cloud.k8s_status("message", ["default"])
    for r in res:
        print(json.dumps(r))

if __name__ == "__main__":
    main()
